from .encode import Encode
from .canvas import Canvas
import numpy as np
from .Table import VersionTable, NormalLocationPoint, AlignmentPatternTable, ECCFomatSiteTable, ECCFomatTable, VersionBitTable, MicroLocationPoint, LocationPointMask
import matplotlib.pyplot as plt


class Qrcode(Encode, Canvas):
    __versionTable = VersionTable
    __NormalLocationPoint = NormalLocationPoint
    __MicroLocationPoint = MicroLocationPoint
    __AlignmentPatternTable = AlignmentPatternTable
    __ECCFormatTable = ECCFomatTable
    __VersionBitTable = VersionBitTable
    __LocationMask = LocationPointMask
    
    def __init__(self, Text):
        super().__init__(Text)
        
    def __dataload(self, version: int, error_correction: str, mask: int = 0, mode = "Normal"):
        self.ECCFomatSiteTable = ECCFomatSiteTable(version)
        self.size = version * 4 + 17
        self._Encode__generate(version, error_correction)
        self._Canvas__resize(Qrcode.__versionTable[str(version)]["moudles"])
        self.QRdataBits = np.full((self.size,  self.size), dtype=int, fill_value=-1)
        self.QRdataRole = np.full((self.size,  self.size), dtype=int, fill_value=0)
        self.Mask = np.full((self.size, self.size), dtype=int, fill_value=0)
        # 0 是整個非定位區域
        # 1 是定位點
        # 2 是分隔符
        # 3 是黑色模塊
        # 4 是定時模塊
        # 5 是對齊模塊
        # 6 是版本模塊
        # 7 是格式模塊
        
        # mask 中
        # 0 為不可更改
        # 1 為 dont care
        
        
    def generate(self, version: int, error_correction: str, mask: int = 0, mode = "Normal"):
        self.__dataload(version, error_correction, mask, mode)
        self.__QrcodeHandler(mode, mask=mask)
        return self.QR
        
    def __SetPositionPattern(self, mode = "Normal"):
        shift = self.size - 7
        PointSet = []
        
        if mode == "Normal":
            PointSet = Qrcode.__NormalLocationPoint
        elif mode == "Micro":
            PointSet = Qrcode.__MicroLocationPoint
            
        for i in PointSet:
            if i[2] == 1:
                self.QRdataRole[i[0]][i[1]] = 1
                self.QRdataRole[i[0] + shift][i[1]] = 1
                self.QRdataRole[i[0]][i[1] + shift] = 1
                self.SetBlack(i[0], i[1])
                self.SetBlack(i[0] + shift, i[1])
                self.SetBlack(i[0], i[1] + shift)
            else:
                self.QRdataRole[i[0]][i[1]] = 1
                self.QRdataRole[i[0] + shift][i[1]] = 1
                self.QRdataRole[i[0]][i[1] + shift] = 1
                self.SetWhite(i[0], i[1])
                self.SetWhite(i[0] + shift, i[1])
                self.SetWhite(i[0], i[1] + shift)
                
        for i in Qrcode.__LocationMask:
            # dont care
            if i[2] == 0:
                self.Mask[i[0] + shift][i[1]] = 1
                self.Mask[i[0]][i[1] + shift] = 1
                self.Mask[i[0]][i[1]] = 1
    
    def __SetSeparatorPattern(self):
        for i in range(8):
            self.QRdataRole[7][i] = 2
            self.QRdataRole[i][7] = 2
            self.QRdataRole[self.size - 8][i] = 2
            self.QRdataRole[i][self.size - 8] = 2
            self.QRdataRole[7][self.size - 1 - i] = 2
            self.QRdataRole[self.size - 1 - i][7] = 2
            self.SetWhite(7, i)
            self.SetWhite(i, 7)
            self.SetWhite(self.size - 8, i)
            self.SetWhite(i, self.size - 8)
            self.SetWhite(7, self.size - 1 - i)
            self.SetWhite(self.size - 1 - i, 7)
        return
    
    def __SerBlackMoudle(self):
        self.QRdataRole[4 * self.version + 9][8] = 3
        self.SetBlack(4 * self.version + 9, 8)
        return
    
    def __SetTimingPattern(self):
        for i in range(8, self.size - 8):
            self.Mask[i][6] = 1
            self.Mask[6][i] = 1
            if i % 2 == 0:
                self.QRdataRole[6][i] = 4
                self.QRdataRole[i][6] = 4
                self.SetBlack(6, i)
                self.SetBlack(i, 6)
            else:
                self.QRdataRole[6][i] = 4
                self.QRdataRole[i][6] = 4
                self.SetWhite(6, i)
                self.SetWhite(i, 6)
        return
    
    def __SetAlignmentPattern(self):
        for i in Qrcode.__AlignmentPatternTable[str(self.version)]:
            for j in Qrcode.__AlignmentPatternTable[str(self.version)]:
                for k in range(-2, 3):
                    for l in range(-2, 3):
                        if i < 7 and j < 7: continue
                        elif i > self.size - 8 and j < 7: continue
                        elif i < 7 and j > self.size - 8: continue
                        ## dont care
                        self.Mask[i + k][j + l] = 1
                        if k == 0 and l == 0:
                            self.QRdataRole[i][j] = 5
                            self.SetBlack(i, j)
                        elif k == -2 or k == 2 or l == -2 or l == 2:
                            self.QRdataRole[i + k][j + l] = 5
                            self.SetBlack(i + k, j + l)
                        else:
                            self.QRdataRole[i + k][j + l] = 5
                            self.SetWhite(i + k, j + l)
        return
    
    def __SetVersionPattern(self):
        if self.version < 7: return
        versionBit = Qrcode.__VersionBitTable[self.version]
        
        idx = 0
        for i in range(5, -1, -1):
            for j in range(self.size - 9, self.size - 12, -1):
                self.QRdataRole[i][j] = 6
                self.QRdataRole[j][i] = 6
                self.SetBlack(i, j) if versionBit[idx] == "1" else self.SetWhite(i, j)
                self.SetBlack(j, i) if versionBit[idx] == "1" else self.SetWhite(j, i)
                idx += 1
        
        return
    
    def __SetFormatPattern(self, ECC: str, mask: int):
        self.mask = mask
        self.format = ""
        if ECC == "L":
            self.format = "01"
        elif ECC == "M":
            self.format = "00"
        elif ECC == "Q":
            self.format = "11"
        elif ECC == "H":
            self.format = "10"
        self.format += format(mask, 'b').zfill(3)
        for i in self.ECCFomatSiteTable:
            x, y = i.split(',')
            x, y = int(x), int(y)
            if ECCFomatTable[self.format][self.ECCFomatSiteTable[i]] == "1":
                self.QRdataRole[x][y] = 7
                self.SetBlack(x, y)
            else:
                self.QRdataRole[x][y] = 7
                self.SetWhite(x, y)
        return
    
    def __SetDataPattern(self):
        data = self._Encode__code
        # print("code", data)
        length = self.version * 4 + 17
        count = 0
        flag = 0
        self.tagTable = np.full((length, length), dtype=int, fill_value=0)
        # tagTable 為 1 表示此點為 padding
        # tagTable 為 2 表示此點為 data
        # tagTable 為 3 表示此點為 ecc
        # tagTable 為 4 表示此點為 remain bits
        tag_values = {"p": 1, "d": 2, "e": 3, "r": 4}
        # print(len(self._Encode__tag), len(data))
        idx = [i for i in range(length - 1, -1, -2)]
        idx = idx[:len(idx) - 4]
        idx += [5, 3, 1]
        # print(len(data), len(idx))
        
        cc = 0
        for i in idx:
            # print(i)
            if flag == 0:
                for j in range(length - 1, -1, -1):
                    for k in range(i, i - 2, -1):
                        if self.QRdataRole[j][k] != 0:
                            continue
                        if data[count] == "1":
                            self.SetBlack(j, k)
                        else:
                            self.SetWhite(j, k)
                        cc += 1
                        self.QRdataBits[j][k] = count
                        self.tagTable[j][k] = tag_values[self._Encode__tag[count]]
                        self.Mask[j][k] = tag_values[self._Encode__tag[count]] + 1
                        # self.show()
                        count += 1
                flag = (flag + 1) % 2
            else:
                for j in range(0, length, 1):
                    for k in range(i, i - 2, -1):
                        if self.QRdataRole[j][k] != 0:
                            continue
                        if data[count] == "1":
                            self.SetBlack(j, k)
                        else:
                            self.SetWhite(j, k)
                        cc += 1
                        self.QRdataBits[j][k] = count
                        self.tagTable[j][k] = tag_values[self._Encode__tag[count]]
                        self.Mask[j][k] = tag_values[self._Encode__tag[count]] + 1
                        # self.show()
                        count += 1
                flag = (flag + 1) % 2
                
        # print("cc", cc, count)
        # for i in range(length):
        #     for j in range(length):
        #         print(f"{self.QRdataBits[i][j]:08d}", end="     ")
        #     print()
            
            
            
        # print()
        # print()
        # print()
        # print()
        # print()
        # print()
        # print()
        # print()
        # print()
        # print()
        # print()
        return
    
    def GetMask(self, i: int, j: int, mask: int):
        if i < 0 or i >= self.size or j < 0 or j >= self.size:
            raise ValueError("The coordinate is out of range") # pragma: no cover
        bit : int = 1
        if self.mask == 0:
            if (i + j) % 2 == 0: bit = 0
        elif self.mask == 1:
            if i % 2 == 0: bit = 0
        elif self.mask == 2:
            if j % 3 == 0: bit = 0
        elif self.mask == 3:
            if (i + j) % 3 == 0: bit = 0
        elif self.mask == 4:
            if (i // 2 + j // 3) % 2 == 0: bit = 0
        elif self.mask == 5:
            if (i * j) % 2 + (i * j) % 3 == 0: bit = 0
        elif self.mask == 6:
            if ((i * j) % 2 + (i * j) % 3) % 2 == 0: bit = 0
        elif self.mask == 7:
            if ((i + j) % 2 + (i * j) % 3) % 2 == 0: bit = 0
        return bit
    
    def __SetMask(self):
        length = self.version * 4 + 17
        for i in range(length):
            for j in range(length):
                if self.QRdataBits[i][j] >= 0:
                    bit = self.GetMask(i, j, self.mask)
                    old = self.GetColor(i, j)
                    self.SetBlack(i, j) if old ^ bit == 1 else self.SetWhite(i, j)
        return
        
    def showdata(self):
        temp = np.full((self.size, self.size), dtype=int, fill_value=128)
        
        for i in range(len(self.tagTable)):
            for j in range(len(self.tagTable)):
                if self.tagTable[i][j] == 2:
                    temp[i][j] = 255
        temp[0][0] = 0
        # plt.imshow(temp, cmap='gray')
        plt.colorbar()
        plt.show()
        return
    
    def __QrcodeHandler(self, mode = "Normal", mask = 0):
        
        self.__SetPositionPattern(mode)
        self.__SetSeparatorPattern()
        self.__SerBlackMoudle()
        self.__SetTimingPattern()
        self.__SetAlignmentPattern()
        self.__SetVersionPattern()
        self.__SetFormatPattern(self.ECC, mask)
        self.__SetDataPattern()
        self.__SetMask()
        
        return self.QR
    
    def InformationMask(self):
        return self.mask
    
    



    






import unittest
from Qart.qart import Qrcode
from Qart.Error import EncodeError
import shutil
import os
import tempfile
from unittest.mock import patch


class TestQrcode(unittest.TestCase):
    @patch('matplotlib.pyplot.imsave')
    def test_qrcodesave(self, mock_plt_save):
        resource_dir = "tests"
        resource_file = "1.png"
        resource_path = os.path.join(resource_dir, resource_file)

        self.assertTrue(os.path.exists(resource_path), f"Resource file {resource_path} does not exist")

        with tempfile.TemporaryDirectory() as tmpdirname:
            temp_file = os.path.join(tmpdirname, resource_file)

            shutil.copy(resource_path, temp_file)

            with tempfile.NamedTemporaryFile(suffix=".png", delete=True) as temp_output_file:
                Qr = Qrcode("Accepted")
                Qr.generate(6, "L", mask = 0, mode = "Normal")
                Qr = Qrcode("1234")
                Qr.generate(6, "L", mask = 1, mode = "Micro")
                Qr = Qrcode("12345")
                Qr.generate(6, "L", mask = 2, mode = "Normal")
                Qr = Qrcode("/O.O/")
                Qr.generate(6, "H", mask = 3, mode = "Normal")
                Qr = Qrcode("謝")
                Qr.generate(6, "M", mask = 4, mode = "Normal")
                Qr = Qrcode("Accepted")
                Qr.generate(6, "Q", mask = 5, mode = "Normal")
                Qr.save("test.jpg")
                Qr = Qrcode("Accepted")
                Qr.generate(6, "M", mask = 6, mode = "Normal")
                Qr.save("test.png")
                Qr = Qrcode("Accepted")
                Qr.generate(6, "Q", mask = 7, mode = "Normal")
                num = Qr.InformationMask()
                Qr.save()
                self.assertEqual(mock_plt_save.call_count, 3)
            with tempfile.NamedTemporaryFile(suffix=".png", delete=True) as temp_output_file:
                Qr = Qrcode("Acceptedwadawdqawdewrwareawerwarewd")
                Qr.generate(20, "L", mask = 0, mode = "Normal")
                
    @patch('matplotlib.pyplot.show')
    def test_qrcode_data_show(self, mock_plt_show):
        Qr = Qrcode("Accepted")
        Qr.generate(6, "Q", mask = 7, mode = "Normal")
        num = Qr.InformationMask()
        Qr.showdata()
        self.assertEqual(mock_plt_show.call_count, 1)
        
    @patch('matplotlib.pyplot.show')
    def test_qrcode_show(self, mock_plt_show):
        Qr = Qrcode("Accepted")
        Qr.generate(6, "Q", mask = 7, mode = "Normal")
        Qr.show()
        self.assertEqual(mock_plt_show.call_count, 1)
        
    def test_qrcode_tp_numpy(self):
        Qr = Qrcode("Accepted")
        Qr.generate(6, "Q", mask = 7, mode = "Normal")
        Qr.to_numpy()
    def test_encode_error(self):
        with self.assertRaises(EncodeError):
            Qr = Qrcode("한")
            Qr.generate(6, "L", mask = 0, mode = "Normal")
            
    def test_value_error(self):
        with self.assertRaises(ValueError):
            Qr = Qrcode("dwadw")
            Qr.generate(50, "L", mask = 0, mode = "Normal")
        
if __name__ == '__main__':
    unittest.main()


import unittest
from Qart.qart import Qart
import shutil
import os
import tempfile
from unittest.mock import patch


class TestQart(unittest.TestCase):
    @patch('matplotlib.pyplot.imsave')
    def test_Qartsave(self, mock_plt_save):
        resource_dir = "tests"
        resource_file = "1.png"
        resource_path = os.path.join(resource_dir, resource_file)
        self.assertTrue(os.path.exists(resource_path), f"Resource file {resource_path} does not exist")

        with tempfile.TemporaryDirectory() as tmpdirname:
            temp_file = os.path.join(tmpdirname, resource_file)

            shutil.copy(resource_path, temp_file)

            with tempfile.NamedTemporaryFile(suffix=".png", delete=True) as temp_output_file:
                Qr = Qart("Accepted")
                Qr.generate(resource_path, 6, "L", mask = 0, mode = "Normal")
                Qr.save()
                self.assertEqual(mock_plt_save.call_count, 1)
            with tempfile.NamedTemporaryFile(suffix=".png", delete=True) as temp_output_file:
                Qr = Qart("Acceptedwadawdqawdewrwareawerwarewd")
                Qr.generate(resource_path, 10, "L", mask = 0, mode = "Normal")

        
if __name__ == '__main__':
    unittest.main()


import unittest
from Qart.img import Image
import shutil
import os
import tempfile
from unittest.mock import patch


class TestImage(unittest.TestCase):
     @patch('matplotlib.pyplot.imsave')
     def test_imgsave(self, mock_plt_save):
          resource_dir = "tests"
          resource_file = "1.png"
          resource_path = os.path.join(resource_dir, resource_file)

          self.assertTrue(os.path.exists(resource_path), f"Resource file {resource_path} does not exist")
          with tempfile.TemporaryDirectory() as tmpdirname:
               temp_file = os.path.join(tmpdirname, resource_file)


               shutil.copy(resource_path, temp_file)

               with tempfile.NamedTemporaryFile(suffix=".png", delete=True) as temp_output_file:

                    img = Image(temp_file)
                    img.SetModuleNums(20)

                    img.save(temp_output_file.name, mode="Modulebase")
                    img.save(temp_output_file.name, mode="RGB")
                    img.save(temp_output_file.name, mode="Grayscale")
                    img.save(mode="OTSU")

                    self.assertEqual(mock_plt_save.call_count, 4)
               with tempfile.NamedTemporaryFile(suffix=".jpg", delete=True) as temp_output_file:
                    img = Image(temp_file)
                    img.SetModuleNums(20)

                    img.save(temp_output_file.name, mode="Modulebase")
                    img.save(temp_output_file.name, mode="RGB")
                    img.save(temp_output_file.name, mode="Grayscale")
                    img.save(mode="OTSU")

                    self.assertEqual(mock_plt_save.call_count, 8)

     @patch('matplotlib.pyplot.show')
     def test_imgshow(self, mock_plt_show):
          resource_dir = "tests"
          resource_file = "1.png"
          resource_path = os.path.join(resource_dir, resource_file)
          self.assertTrue(os.path.exists(resource_path), f"Resource file {resource_path} does not exist")
          img = Image(resource_path)
          img.SetModuleNums(20)
          img.show()
          img.show("Grayscale")
          img.show("OTSU")
          img.show("Modulebase")
          self.assertEqual(mock_plt_show.call_count, 4)

          
if __name__ == '__main__':
     unittest.main()


import numpy as np
import matplotlib.pyplot as plt
import time

class Canvas:
    
    def __init__(self):
        return # pragma: no cover
        
    def __resize(self, size: int):
        self.QR = np.full((size, size), dtype=int, fill_value=128)
        self.size = size
        self.version = (size - 21) // 4 + 1
        return
    
    def SetBlack(self, x: int, y: int):
        if(x < 0 or x >= len(self.QR) or y < 0 or y >= len(self.QR)):
            raise ValueError("The coordinate is out of range") # pragma: no cover
        self.QR[x][y] = 0
        return
    
    def SetWhite(self, x: int, y: int):
        if(x < 0 or x >= len(self.QR) or y < 0 or y >= len(self.QR)):
            raise ValueError("The coordinate is out of range") # pragma: no cover
        self.QR[x][y] = 255
        return
    
    def GetColor(self, x: int, y: int):
        if(x < 0 or x >= len(self.QR) or y < 0 or y >= len(self.QR)):
            raise ValueError("The coordinate is out of range") # pragma: no cover
        return 0 if self.QR[x][y] == 0 else 1
    
    def show(self):
        plt.imshow(self.QR, cmap='gray')  # 'cmap' 參數控制著色映射，'gray' 表示灰階
        plt.colorbar()  # 顯示色條
        plt.show()
        
                 
    def save(self, filename: str = "", padding: int = 0, module_size: int = 8):
        if(filename == ""):
            filename = time.strftime("%Y%m%d%H%M%S", time.localtime())
        if(filename[-4:] == ".png"):
            filename = filename[:-4]
        if(filename[-4:] == ".jpg"):
            filename = filename[:-4]
        pixel_num = (self.size + padding * 2) * module_size
        image = np.full((pixel_num, pixel_num), dtype=int, fill_value=255)
        for i in range(self.size):
            for j in range(self.size):
                if(self.QR[i][j] == 0):
                    image[(i + padding) * module_size: (i + padding + 1) * module_size, (j + padding) * module_size: (j + padding + 1) * module_size] = 0
        plt.imsave(filename + ".png", image, cmap='gray')
        return
    
    def to_numpy(self, module_size: int = 8, padding: int = 0):
        pixel_num = (self.size) * module_size + padding * 2
        image = np.full((pixel_num, pixel_num), dtype=int, fill_value=255)
        for i in range(self.size):
            for j in range(self.size):
                if(self.QR[i][j] == 0):
                    image[(i) * module_size + padding: (i + 1) * module_size + padding, (j) * module_size + padding: (j + 1) * module_size + padding] = 0
        return image
    
    
    


import cv2
import numpy as np
import sys
import math
import matplotlib.pyplot as plt
import time


class Img:
    @classmethod
    def read(cls, path):
        return cv2.cvtColor(cv2.imread(path), cv2.COLOR_BGR2RGB)
    
    @classmethod
    def Grayscale(cls, img):
        grayscale_img = np.empty(img.shape[:-1])  # 假設 img 是 (高, 寬, 3)
        grayscale_img = np.dot(img[...,:3], [0.299, 0.587, 0.114]).round().astype(np.int32)
        return grayscale_img
    
    @classmethod
    def OTSU(cls, img_luminance):
        img_luminance = img_luminance.astype(np.uint8)
        histogram = np.zeros(256)
        for i in range(img_luminance.shape[0]):
            for j in range(img_luminance.shape[1]):
                histogram[img_luminance[i][j]] += 1
        distribution = histogram / np.sum(histogram)

        best_threshold = 0
        minimum_variance = sys.maxsize

        for threshold in range(256):
            # calculate mean
            # --------------
            # lower than threshold
            sum_low = 0
            for i in range(threshold):
                sum_low += distribution[i] * i
            mean_low = sum_low / np.sum(distribution[:threshold])

            # higher than threshold
            sum_high = 0
            for i in range(threshold, 256):
                sum_high += distribution[i] * i
            mean_high = sum_high / np.sum(distribution[threshold:])

            # calculate variance
            # ------------------
            # lower
            variance_low = 0
            for i in range(threshold):
                variance_low += (mean_low - i)**2 * distribution[i]

            # higher
            variance_high = 0
            for i in range(threshold, 256):
                variance_high += (mean_high - i)**2 * distribution[i]

            # sum up
            group_in_variance = variance_low + variance_high

            if group_in_variance < minimum_variance:
                best_threshold = threshold
                minimum_variance = group_in_variance

        _, binarized_img = cv2.threshold(
            img_luminance, best_threshold, 255, cv2.THRESH_BINARY)
        return binarized_img, best_threshold, minimum_variance

    @classmethod
    def module_based_binarization(cls, img, modulenums):
        
        module_size = Img.__calculateMoudleSize(len(img), modulenums)
        img_size = img.shape[0]
        subimage_size = module_size
        binary_img = np.zeros(
            (img.shape[0] // module_size, img.shape[1] // module_size), dtype=np.uint8)

        # 自己建立Gaussian kernel
        gaussian_kernel = np.zeros((subimage_size, subimage_size))
        center = (subimage_size - 1) // 2
        sigma = 1

        # 設定Gaussian kernel內各個i, j的值
        for i in range(subimage_size):
            for j in range(subimage_size):
                gaussian_kernel[i][j] = 1/(2*math.pi*sigma**2) * \
                    np.exp(-((i-center)**2 + (j-center)**2) / 2*sigma**2)

        # 讓此kernel的總和為1
        gaussian_kernel /= np.sum(gaussian_kernel)

        # 計算各個subimage的binarization的結果後
        # assign給整個subimage，即Module
        for i in range(img_size // subimage_size):
            for j in range(img_size // subimage_size):
                try:
                    region = img[i*subimage_size:(i+1)*subimage_size,
                                j*subimage_size:(j+1)*subimage_size]
                    value = np.round(np.sum(region * gaussian_kernel) / 255)
                    binary_img[i][j] = value * 255
                except:
                    pass
        
        if modulenums > len(binary_img):
            ## copilot 寫的 不知道能不能動
            ## 要把圖片放大時在下方及右方補 0
            binary_img = np.pad(binary_img, ((0, modulenums - len(binary_img)), (0, 0)), 'constant', constant_values=(255))      
        else:
            binary_img = binary_img[:modulenums, :modulenums]
        return binary_img
    
    @classmethod
    def __calculateMoudleSize(cls, img_size, qrMoudlesize):
        return img_size // qrMoudlesize
    
    @classmethod
    def image2moudlebase(cls, moudlesnum, path: str):
        image = Img.read(path)
        image = Img.Grayscale(image)
        image = Img.OTSU(image)[0]
        image = Img.module_based_binarization(image, moudlesnum)
        return image
    
    @classmethod
    def imagemoudlebase(cls, moudlesnum, image: np.ndarray):
        image = Img.Grayscale(image)
        image = Img.OTSU(image)[0]
        image = Img.module_based_binarization(image, moudlesnum)
        return image
    
class Image(Img):
    def __init__(self, path: str):
        self.path = path
        self.image = self.read(path)
        self.grayscale = self.Grayscale(self.image)
        self.otsu = self.OTSU(self.grayscale)[0]
        self.modulebase = self.module_based_binarization(self.otsu, 50)
        return
    
    def SetModuleNums(self, moudlesnum):
        self.modulebase = self.module_based_binarization(self.otsu, moudlesnum)
        return
    
    def show(self, mode = "RGB"):
        if mode == "RGB":
            plt.imshow(self.image)
        elif mode == "Grayscale":
            plt.imshow(self.grayscale, cmap='gray')
        elif mode == "OTSU":
            plt.imshow(self.otsu, cmap='gray')
        elif mode == "Modulebase":
            plt.imshow(self.modulebase, cmap='gray')
        plt.show()
        
    def save(self, filename: str = "", mode: str = "RGB"):
        if filename == "":
            filename = time.strftime("%Y%m%d%H%M%S", time.localtime())
        if filename[-4:] == ".png":
            filename = filename[:-4]
        if filename[-4:] == ".jpg":
            filename = filename[:-4]
            
        if mode == "RGB":
            plt.imsave(filename + ".png", self.image)
        elif mode == "Grayscale":
            plt.imsave(filename + ".png", self.grayscale, cmap='gray')
        elif mode == "OTSU":
            plt.imsave(filename + ".png", self.otsu, cmap='gray')
        elif mode == "Modulebase":
            pixel_num = self.image.shape[0]
            rate = pixel_num // len(self.modulebase)
            pixel_num = len(self.modulebase) * rate
            image = np.full((pixel_num, pixel_num), dtype=int, fill_value=255)
            for i in range(len(self.modulebase)):
                for j in range(len(self.modulebase)):
                    if(self.modulebase[i][j] == 0):
                        image[(i) * rate: (i + 1) * rate, (j) * rate: (j + 1) * rate] = 0
            plt.imsave(filename + ".png", image, cmap='gray')
        
            


from setuptools import setup, find_packages

with open('requirements.txt') as f:
    required = f.read().splitlines()

setup(
    name="Qart",
    version="0.0.20",
    description="A short description of your package",
    long_description=open("README.md").read(),
    long_description_content_type="text/markdown",
    url="https://github.com/as6325400/Qart",
    author="as6325400",
    author_email="as6325400@gmail.com",
    license="MIT",
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=required,  
)
